var __getOwnPropNames = Object.getOwnPropertyNames;
var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});
var __commonJS = (cb, mod) => function __require2() {
  return mod || (0, cb[__getOwnPropNames(cb)[0]])((mod = { exports: {} }).exports, mod), mod.exports;
};

// src/index.ts
import { readFileSync } from "fs";
import { cloneDeep, endsWith, merge } from "lodash";
import { dirname as dirname2 } from "path";

// src/formatter.ts
import { format as prettify } from "prettier";
function format(code, options) {
  if (!options.format) {
    return code;
  }
  return prettify(code, { parser: "typescript", ...options.style });
}

// src/generator.ts
import { memoize as memoize2, omit } from "lodash";

// src/types/AST.ts
function hasComment(ast) {
  return "comment" in ast && ast.comment != null && ast.comment !== "" || // Compare to true because ast.deprecated might be undefined
  "deprecated" in ast && ast.deprecated === true;
}
function hasStandaloneName(ast) {
  return "standaloneName" in ast && ast.standaloneName != null && ast.standaloneName !== "";
}
var T_ANY = {
  type: "ANY"
};
var T_ANY_ADDITIONAL_PROPERTIES = {
  keyName: "[k: string]",
  type: "ANY"
};
var T_UNKNOWN = {
  type: "UNKNOWN"
};
var T_UNKNOWN_ADDITIONAL_PROPERTIES = {
  keyName: "[k: string]",
  type: "UNKNOWN"
};

// src/utils.ts
import { deburr, isPlainObject as isPlainObject2, trim, upperFirst } from "lodash";
import { basename, dirname, extname, normalize, sep, posix } from "path";

// src/types/JSONSchema.ts
import { isPlainObject, memoize } from "lodash";
var Parent = Symbol("Parent");
var getRootSchema = memoize((schema) => {
  const parent = schema[Parent];
  if (!parent) {
    return schema;
  }
  return getRootSchema(parent);
});
function isBoolean(schema) {
  return schema === true || schema === false;
}
function isPrimitive(schema) {
  return !isPlainObject(schema);
}
function isCompound(schema) {
  return Array.isArray(schema.type) || "anyOf" in schema || "oneOf" in schema;
}

// src/utils.ts
function Try(fn, err) {
  try {
    return fn();
  } catch (e) {
    return err(e);
  }
}
var BLACKLISTED_KEYS = /* @__PURE__ */ new Set([
  "id",
  "$defs",
  "$id",
  "$schema",
  "title",
  "description",
  "default",
  "multipleOf",
  "maximum",
  "exclusiveMaximum",
  "minimum",
  "exclusiveMinimum",
  "maxLength",
  "minLength",
  "pattern",
  "additionalItems",
  "items",
  "maxItems",
  "minItems",
  "uniqueItems",
  "maxProperties",
  "minProperties",
  "required",
  "additionalProperties",
  "definitions",
  "properties",
  "patternProperties",
  "dependencies",
  "enum",
  "type",
  "allOf",
  "anyOf",
  "oneOf",
  "not"
]);
function traverseObjectKeys(obj, callback, processed) {
  Object.keys(obj).forEach((k) => {
    if (obj[k] && typeof obj[k] === "object" && !Array.isArray(obj[k])) {
      traverse(obj[k], callback, processed, k);
    }
  });
}
function traverseArray(arr, callback, processed) {
  arr.forEach((s, k) => traverse(s, callback, processed, k.toString()));
}
function traverse(schema, callback, processed = /* @__PURE__ */ new Set(), key) {
  if (processed.has(schema)) {
    return;
  }
  processed.add(schema);
  callback(schema, key ?? null);
  if (schema.anyOf) {
    traverseArray(schema.anyOf, callback, processed);
  }
  if (schema.allOf) {
    traverseArray(schema.allOf, callback, processed);
  }
  if (schema.oneOf) {
    traverseArray(schema.oneOf, callback, processed);
  }
  if (schema.properties) {
    traverseObjectKeys(schema.properties, callback, processed);
  }
  if (schema.patternProperties) {
    traverseObjectKeys(schema.patternProperties, callback, processed);
  }
  if (schema.additionalProperties && typeof schema.additionalProperties === "object") {
    traverse(schema.additionalProperties, callback, processed);
  }
  if (schema.items) {
    const { items } = schema;
    if (Array.isArray(items)) {
      traverseArray(items, callback, processed);
    } else {
      traverse(items, callback, processed);
    }
  }
  if (schema.additionalItems && typeof schema.additionalItems === "object") {
    traverse(schema.additionalItems, callback, processed);
  }
  if (schema.dependencies) {
    if (Array.isArray(schema.dependencies)) {
      traverseArray(schema.dependencies, callback, processed);
    } else {
      traverseObjectKeys(schema.dependencies, callback, processed);
    }
  }
  if (schema.definitions) {
    traverseObjectKeys(schema.definitions, callback, processed);
  }
  if (schema.$defs) {
    traverseObjectKeys(schema.$defs, callback, processed);
  }
  if (schema.not) {
    traverse(schema.not, callback, processed);
  }
  Object.keys(schema).filter((key2) => !BLACKLISTED_KEYS.has(key2)).forEach((key2) => {
    const child = schema[key2];
    if (child && typeof child === "object") {
      traverseObjectKeys(child, callback, processed);
    }
  });
}
function justName(filename = "") {
  return stripExtension(basename(filename));
}
function stripExtension(filename) {
  return filename.replace(extname(filename), "");
}
function toSafeString(string) {
  return upperFirst(
    // remove accents, umlauts, ... by their basic latin letters
    deburr(string).replace(/(^\s*[^a-zA-Z_$])|([^a-zA-Z_$\d])/g, " ").replace(/^_[a-z]/g, (match) => match.toUpperCase()).replace(/_[a-z]/g, (match) => match.substr(1, match.length).toUpperCase()).replace(/([\d$]+[a-zA-Z])/g, (match) => match.toUpperCase()).replace(/\s+([a-zA-Z])/g, (match) => trim(match.toUpperCase())).replace(/\s/g, "")
  );
}
function generateName(from, usedNames) {
  let name = toSafeString(from);
  if (!name) {
    name = "NoName";
  }
  if (usedNames.has(name)) {
    let counter = 1;
    let nameWithCounter = `${name}${counter}`;
    while (usedNames.has(nameWithCounter)) {
      nameWithCounter = `${name}${counter}`;
      counter++;
    }
    name = nameWithCounter;
  }
  usedNames.add(name);
  return name;
}
function error(...messages) {
  if (!process.env.VERBOSE) {
    return console.error(messages);
  }
  console.error(getStyledTextForLogging("red")?.("error"), ...messages);
}
function log(style, title, ...messages) {
  if (!process.env.VERBOSE) {
    return;
  }
  let lastMessage = null;
  if (messages.length > 1 && typeof messages[messages.length - 1] !== "string") {
    lastMessage = messages.splice(messages.length - 1, 1);
  }
  console.info(__require("cli-color").whiteBright.bgCyan("debug"), getStyledTextForLogging(style)?.(title), ...messages);
  if (lastMessage) {
    console.dir(lastMessage, { depth: 6, maxArrayLength: 6 });
  }
}
function getStyledTextForLogging(style) {
  if (!process.env.VERBOSE) {
    return;
  }
  switch (style) {
    case "blue":
      return __require("cli-color").whiteBright.bgBlue;
    case "cyan":
      return __require("cli-color").whiteBright.bgCyan;
    case "green":
      return __require("cli-color").whiteBright.bgGreen;
    case "magenta":
      return __require("cli-color").whiteBright.bgMagenta;
    case "red":
      return __require("cli-color").whiteBright.bgRedBright;
    case "white":
      return __require("cli-color").black.bgWhite;
    case "yellow":
      return __require("cli-color").whiteBright.bgYellow;
  }
}
function escapeBlockComment(schema) {
  const replacer = "* /";
  if (schema === null || typeof schema !== "object") {
    return;
  }
  for (const key of Object.keys(schema)) {
    if (key === "description" && typeof schema[key] === "string") {
      schema[key] = schema[key].replace(/\*\//g, replacer);
    }
  }
}
function pathTransform(outputPath, inputPath, filePath) {
  const inPathList = normalize(inputPath).split(sep);
  const filePathList = dirname(normalize(filePath)).split(sep);
  const filePathRel = filePathList.filter((f, i) => f !== inPathList[i]);
  return posix.join(posix.normalize(outputPath), ...filePathRel);
}
function maybeStripDefault(schema) {
  if (!("default" in schema)) {
    return schema;
  }
  switch (schema.type) {
    case "array":
      if (Array.isArray(schema.default)) {
        return schema;
      }
      break;
    case "boolean":
      if (typeof schema.default === "boolean") {
        return schema;
      }
      break;
    case "integer":
    case "number":
      if (typeof schema.default === "number") {
        return schema;
      }
      break;
    case "string":
      if (typeof schema.default === "string") {
        return schema;
      }
      break;
    case "null":
      if (schema.default === null) {
        return schema;
      }
      break;
    case "object":
      if (isPlainObject2(schema.default)) {
        return schema;
      }
      break;
  }
  delete schema.default;
  return schema;
}
function maybeStripNameHints(schema) {
  if ("$id" in schema) {
    delete schema.$id;
  }
  if ("description" in schema) {
    delete schema.description;
  }
  if ("name" in schema) {
    delete schema.name;
  }
  return schema;
}
function appendToDescription(existingDescription, ...values) {
  if (existingDescription) {
    return `${existingDescription}

${values.join("\n")}`;
  }
  return values.join("\n");
}
function isSchemaLike(schema) {
  if (!isPlainObject2(schema)) {
    return false;
  }
  const parent = schema[Parent];
  if (parent === null) {
    return true;
  }
  const JSON_SCHEMA_KEYWORDS = [
    "$defs",
    "allOf",
    "anyOf",
    "definitions",
    "dependencies",
    "enum",
    "not",
    "oneOf",
    "patternProperties",
    "properties",
    "required"
  ];
  if (JSON_SCHEMA_KEYWORDS.some((_) => parent[_] === schema)) {
    return false;
  }
  return true;
}

// src/generator.ts
function generate(ast, options = DEFAULT_OPTIONS) {
  return [
    options.bannerComment,
    declareNamedTypes(ast, options, ast.standaloneName),
    declareNamedInterfaces(ast, options, ast.standaloneName),
    declareEnums(ast, options)
  ].filter(Boolean).join("\n\n") + "\n";
}
function declareEnums(ast, options, processed = /* @__PURE__ */ new Set()) {
  if (processed.has(ast)) {
    return "";
  }
  processed.add(ast);
  let type = "";
  switch (ast.type) {
    case "ENUM":
      return generateStandaloneEnum(ast, options) + "\n";
    case "ARRAY":
      return declareEnums(ast.params, options, processed);
    case "UNION":
    case "INTERSECTION":
      return ast.params.reduce((prev, ast2) => prev + declareEnums(ast2, options, processed), "");
    case "TUPLE":
      type = ast.params.reduce((prev, ast2) => prev + declareEnums(ast2, options, processed), "");
      if (ast.spreadParam) {
        type += declareEnums(ast.spreadParam, options, processed);
      }
      return type;
    case "INTERFACE":
      return getSuperTypesAndParams(ast).reduce((prev, ast2) => prev + declareEnums(ast2, options, processed), "");
    default:
      return "";
  }
}
function declareNamedInterfaces(ast, options, rootASTName, processed = /* @__PURE__ */ new Set()) {
  if (processed.has(ast)) {
    return "";
  }
  processed.add(ast);
  let type = "";
  switch (ast.type) {
    case "ARRAY":
      type = declareNamedInterfaces(ast.params, options, rootASTName, processed);
      break;
    case "INTERFACE":
      type = [
        hasStandaloneName(ast) && (ast.standaloneName === rootASTName || options.declareExternallyReferenced) && generateStandaloneInterface(ast, options),
        getSuperTypesAndParams(ast).map((ast2) => declareNamedInterfaces(ast2, options, rootASTName, processed)).filter(Boolean).join("\n")
      ].filter(Boolean).join("\n");
      break;
    case "INTERSECTION":
    case "TUPLE":
    case "UNION":
      type = ast.params.map((_) => declareNamedInterfaces(_, options, rootASTName, processed)).filter(Boolean).join("\n");
      if (ast.type === "TUPLE" && ast.spreadParam) {
        type += declareNamedInterfaces(ast.spreadParam, options, rootASTName, processed);
      }
      break;
    default:
      type = "";
  }
  return type;
}
function declareNamedTypes(ast, options, rootASTName, processed = /* @__PURE__ */ new Set()) {
  if (processed.has(ast)) {
    return "";
  }
  processed.add(ast);
  switch (ast.type) {
    case "ARRAY":
      return [
        declareNamedTypes(ast.params, options, rootASTName, processed),
        hasStandaloneName(ast) ? generateStandaloneType(ast, options) : void 0
      ].filter(Boolean).join("\n");
    case "ENUM":
      return "";
    case "INTERFACE":
      return getSuperTypesAndParams(ast).map(
        (ast2) => (ast2.standaloneName === rootASTName || options.declareExternallyReferenced) && declareNamedTypes(ast2, options, rootASTName, processed)
      ).filter(Boolean).join("\n");
    case "INTERSECTION":
    case "TUPLE":
    case "UNION":
      return [
        hasStandaloneName(ast) ? generateStandaloneType(ast, options) : void 0,
        ast.params.map((ast2) => declareNamedTypes(ast2, options, rootASTName, processed)).filter(Boolean).join("\n"),
        "spreadParam" in ast && ast.spreadParam ? declareNamedTypes(ast.spreadParam, options, rootASTName, processed) : void 0
      ].filter(Boolean).join("\n");
    default:
      if (hasStandaloneName(ast)) {
        return generateStandaloneType(ast, options);
      }
      return "";
  }
}
function generateTypeUnmemoized(ast, options) {
  const type = generateRawType(ast, options);
  if (options.strictIndexSignatures && ast.keyName === "[k: string]") {
    return `${type} | undefined`;
  }
  return type;
}
var generateType = memoize2(generateTypeUnmemoized);
function generateRawType(ast, options) {
  log("magenta", "generator", ast);
  if (hasStandaloneName(ast)) {
    return toSafeString(ast.standaloneName);
  }
  switch (ast.type) {
    case "ANY":
      return "any";
    case "ARRAY":
      return (() => {
        const type = generateType(ast.params, options);
        return type.endsWith('"') ? "(" + type + ")[]" : type + "[]";
      })();
    case "BOOLEAN":
      return "boolean";
    case "INTERFACE":
      return generateInterface(ast, options);
    case "INTERSECTION":
      return generateSetOperation(ast, options);
    case "LITERAL":
      return JSON.stringify(ast.params);
    case "NEVER":
      return "never";
    case "NUMBER":
      return "number";
    case "NULL":
      return "null";
    case "OBJECT":
      return "object";
    case "REFERENCE":
      return ast.params;
    case "STRING":
      return "string";
    case "TUPLE":
      return (() => {
        const minItems = ast.minItems;
        const maxItems = ast.maxItems || -1;
        let spreadParam = ast.spreadParam;
        const astParams = [...ast.params];
        if (minItems > 0 && minItems > astParams.length && ast.spreadParam === void 0) {
          if (maxItems < 0) {
            spreadParam = options.unknownAny ? T_UNKNOWN : T_ANY;
          }
        }
        if (maxItems > astParams.length && ast.spreadParam === void 0) {
          for (let i = astParams.length; i < maxItems; i += 1) {
            astParams.push(options.unknownAny ? T_UNKNOWN : T_ANY);
          }
        }
        function addSpreadParam(params) {
          if (spreadParam) {
            const spread = "...(" + generateType(spreadParam, options) + ")[]";
            params.push(spread);
          }
          return params;
        }
        function paramsToString(params) {
          return "[" + params.join(", ") + "]";
        }
        const paramsList = astParams.map((param) => generateType(param, options));
        if (paramsList.length > minItems) {
          const cumulativeParamsList = paramsList.slice(0, minItems);
          const typesToUnion = [];
          if (cumulativeParamsList.length > 0) {
            typesToUnion.push(paramsToString(cumulativeParamsList));
          } else {
            typesToUnion.push(paramsToString([]));
          }
          for (let i = minItems; i < paramsList.length; i += 1) {
            cumulativeParamsList.push(paramsList[i]);
            if (i === paramsList.length - 1) {
              addSpreadParam(cumulativeParamsList);
            }
            typesToUnion.push(paramsToString(cumulativeParamsList));
          }
          return typesToUnion.join("|");
        }
        return paramsToString(addSpreadParam(paramsList));
      })();
    case "UNION":
      return generateSetOperation(ast, options);
    case "UNKNOWN":
      return "unknown";
    case "CUSTOM_TYPE":
      return ast.params;
  }
}
function generateSetOperation(ast, options) {
  const members = ast.params.map((_) => generateType(_, options));
  const separator = ast.type === "UNION" ? "|" : "&";
  return members.length === 1 ? members[0] : "(" + members.join(" " + separator + " ") + ")";
}
function generateInterface(ast, options) {
  return `{
` + ast.params.filter((_) => !_.isPatternProperty && !_.isUnreachableDefinition).map(
    ({ isRequired, keyName, ast: ast2 }) => [isRequired, keyName, ast2, generateType(ast2, options)]
  ).map(
    ([isRequired, keyName, ast2, type]) => (hasComment(ast2) && !ast2.standaloneName ? generateComment(ast2.comment, ast2.deprecated) + "\n" : "") + escapeKeyName(keyName) + (isRequired ? "" : "?") + ": " + type
  ).join("\n") + "\n}";
}
function generateComment(comment, deprecated) {
  const commentLines = ["/**"];
  if (deprecated) {
    commentLines.push(" * @deprecated");
  }
  if (typeof comment !== "undefined") {
    commentLines.push(...comment.split("\n").map((_) => " * " + _));
  }
  commentLines.push(" */");
  return commentLines.join("\n");
}
function generateStandaloneEnum(ast, options) {
  return (hasComment(ast) ? generateComment(ast.comment, ast.deprecated) + "\n" : "") + "export " + (options.enableConstEnums ? "const " : "") + `enum ${toSafeString(ast.standaloneName)} {
` + ast.params.map(({ ast: ast2, keyName }) => keyName + " = " + generateType(ast2, options)).join(",\n") + "\n}";
}
function generateStandaloneInterface(ast, options) {
  return (hasComment(ast) ? generateComment(ast.comment, ast.deprecated) + "\n" : "") + `export interface ${toSafeString(ast.standaloneName)} ` + (ast.superTypes.length > 0 ? `extends ${ast.superTypes.map((superType) => toSafeString(superType.standaloneName)).join(", ")} ` : "") + generateInterface(ast, options);
}
function generateStandaloneType(ast, options) {
  return (hasComment(ast) ? generateComment(ast.comment) + "\n" : "") + `export type ${toSafeString(ast.standaloneName)} = ${generateType(
    omit(ast, "standaloneName"),
    options
  )}`;
}
function escapeKeyName(keyName) {
  if (keyName.length && /[A-Za-z_$]/.test(keyName.charAt(0)) && /^[\w$]+$/.test(keyName)) {
    return keyName;
  }
  if (keyName === "[k: string]") {
    return keyName;
  }
  return JSON.stringify(keyName);
}
function getSuperTypesAndParams(ast) {
  return ast.params.map((param) => param.ast).concat(ast.superTypes);
}

// src/normalizer.ts
import { isDeepStrictEqual } from "util";
var rules = /* @__PURE__ */ new Map();
function hasType(schema, type) {
  return schema.type === type || Array.isArray(schema.type) && schema.type.includes(type);
}
function isObjectType(schema) {
  return schema.properties !== void 0 || hasType(schema, "object") || hasType(schema, "any");
}
function isArrayType(schema) {
  return schema.items !== void 0 || hasType(schema, "array") || hasType(schema, "any");
}
rules.set('Remove `type=["null"]` if `enum=[null]`', (schema) => {
  if (Array.isArray(schema.enum) && schema.enum.some((e) => e === null) && Array.isArray(schema.type) && schema.type.includes("null")) {
    schema.type = schema.type.filter((type) => type !== "null");
  }
});
rules.set("Destructure unary types", (schema) => {
  if (schema.type && Array.isArray(schema.type) && schema.type.length === 1) {
    schema.type = schema.type[0];
  }
});
rules.set("Add empty `required` property if none is defined", (schema) => {
  if (isObjectType(schema) && !("required" in schema)) {
    schema.required = [];
  }
});
rules.set("Transform `required`=false to `required`=[]", (schema) => {
  if (schema.required === false) {
    schema.required = [];
  }
});
rules.set("Default additionalProperties", (schema, _, options) => {
  if (isObjectType(schema) && !("additionalProperties" in schema) && schema.patternProperties === void 0) {
    schema.additionalProperties = options.additionalProperties;
  }
});
rules.set("Transform id to $id", (schema, fileName) => {
  if (!isSchemaLike(schema)) {
    return;
  }
  if (schema.id && schema.$id && schema.id !== schema.$id) {
    throw ReferenceError(
      `Schema must define either id or $id, not both. Given id=${schema.id}, $id=${schema.$id} in ${fileName}`
    );
  }
  if (schema.id) {
    schema.$id = schema.id;
    delete schema.id;
  }
});
rules.set("Add an $id to anything that needs it", (schema, fileName, _options, _key, dereferencedPaths) => {
  if (!isSchemaLike(schema)) {
    return;
  }
  if (!schema.$id && !schema[Parent]) {
    schema.$id = toSafeString(justName(fileName));
    return;
  }
  if (!isArrayType(schema) && !isObjectType(schema)) {
    return;
  }
  const dereferencedName = dereferencedPaths.get(schema);
  if (!schema.$id && !schema.title && dereferencedName) {
    schema.$id = toSafeString(justName(dereferencedName));
  }
  if (dereferencedName) {
    dereferencedPaths.delete(schema);
  }
});
rules.set("Escape closing JSDoc comment", (schema) => {
  escapeBlockComment(schema);
});
rules.set("Add JSDoc comments for minItems and maxItems", (schema) => {
  if (!isArrayType(schema)) {
    return;
  }
  const commentsToAppend = [
    "minItems" in schema ? `@minItems ${schema.minItems}` : "",
    "maxItems" in schema ? `@maxItems ${schema.maxItems}` : ""
  ].filter(Boolean);
  if (commentsToAppend.length) {
    schema.description = appendToDescription(schema.description, ...commentsToAppend);
  }
});
rules.set("Optionally remove maxItems and minItems", (schema, _fileName, options) => {
  if (!isArrayType(schema)) {
    return;
  }
  if ("minItems" in schema && options.ignoreMinAndMaxItems) {
    delete schema.minItems;
  }
  if ("maxItems" in schema && (options.ignoreMinAndMaxItems || options.maxItems === -1)) {
    delete schema.maxItems;
  }
});
rules.set("Normalize schema.minItems", (schema, _fileName, options) => {
  if (options.ignoreMinAndMaxItems) {
    return;
  }
  if (!isArrayType(schema)) {
    return;
  }
  const { minItems } = schema;
  schema.minItems = typeof minItems === "number" ? minItems : 0;
});
rules.set("Remove maxItems if it is big enough to likely cause OOMs", (schema, _fileName, options) => {
  if (options.ignoreMinAndMaxItems || options.maxItems === -1) {
    return;
  }
  if (!isArrayType(schema)) {
    return;
  }
  const { maxItems, minItems } = schema;
  if (maxItems !== void 0 && maxItems - minItems > options.maxItems) {
    delete schema.maxItems;
  }
});
rules.set("Normalize schema.items", (schema, _fileName, options) => {
  if (options.ignoreMinAndMaxItems) {
    return;
  }
  const { maxItems, minItems } = schema;
  const hasMaxItems = typeof maxItems === "number" && maxItems >= 0;
  const hasMinItems = typeof minItems === "number" && minItems > 0;
  if (schema.items && !Array.isArray(schema.items) && (hasMaxItems || hasMinItems)) {
    const items = schema.items;
    const newItems = Array(maxItems || minItems || 0).fill(items);
    if (!hasMaxItems) {
      schema.additionalItems = items;
    }
    schema.items = newItems;
  }
  if (Array.isArray(schema.items) && hasMaxItems && maxItems < schema.items.length) {
    schema.items = schema.items.slice(0, maxItems);
  }
  return schema;
});
rules.set("Remove extends, if it is empty", (schema) => {
  if (!schema.hasOwnProperty("extends")) {
    return;
  }
  if (schema.extends == null || Array.isArray(schema.extends) && schema.extends.length === 0) {
    delete schema.extends;
  }
});
rules.set("Make extends always an array, if it is defined", (schema) => {
  if (schema.extends == null) {
    return;
  }
  if (!Array.isArray(schema.extends)) {
    schema.extends = [schema.extends];
  }
});
rules.set("Transform definitions to $defs", (schema, fileName) => {
  if (schema.definitions && schema.$defs && !isDeepStrictEqual(schema.definitions, schema.$defs)) {
    throw ReferenceError(
      `Schema must define either definitions or $defs, not both. Given id=${schema.id} in ${fileName}`
    );
  }
  if (schema.definitions) {
    schema.$defs = schema.definitions;
    delete schema.definitions;
  }
});
rules.set("Transform const to singleton enum", (schema) => {
  if (schema.const !== void 0) {
    schema.enum = [schema.const];
    delete schema.const;
  }
});
function normalize2(rootSchema, dereferencedPaths, filename, options) {
  rules.forEach((rule) => traverse(rootSchema, (schema, key) => rule(schema, filename, options, key, dereferencedPaths)));
  return rootSchema;
}

// src/optimizer.ts
import { uniqBy } from "lodash";
function optimize(ast, options, processed = /* @__PURE__ */ new Set()) {
  if (processed.has(ast)) {
    return ast;
  }
  processed.add(ast);
  switch (ast.type) {
    case "INTERFACE":
      return Object.assign(ast, {
        params: ast.params.map((_) => Object.assign(_, { ast: optimize(_.ast, options, processed) }))
      });
    case "INTERSECTION":
    case "UNION":
      const optimizedAST = Object.assign(ast, {
        params: ast.params.map((_) => optimize(_, options, processed))
      });
      if (optimizedAST.params.some((_) => _.type === "ANY")) {
        log("cyan", "optimizer", "[A, B, C, Any] -> Any", optimizedAST);
        return T_ANY;
      }
      if (optimizedAST.params.some((_) => _.type === "UNKNOWN")) {
        log("cyan", "optimizer", "[A, B, C, Unknown] -> Unknown", optimizedAST);
        return T_UNKNOWN;
      }
      if (optimizedAST.params.every((_) => {
        const a = generateType(omitStandaloneName(_), options);
        const b = generateType(omitStandaloneName(optimizedAST.params[0]), options);
        return a === b;
      }) && optimizedAST.params.some((_) => _.standaloneName !== void 0)) {
        log("cyan", "optimizer", "[A (named), A] -> [A (named)]", optimizedAST);
        optimizedAST.params = optimizedAST.params.filter((_) => _.standaloneName !== void 0);
      }
      const params = uniqBy(optimizedAST.params, (_) => generateType(_, options));
      if (params.length !== optimizedAST.params.length) {
        log("cyan", "optimizer", "[A, B, B] -> [A, B]", optimizedAST);
        optimizedAST.params = params;
      }
      return Object.assign(optimizedAST, {
        params: optimizedAST.params.map((_) => optimize(_, options, processed))
      });
    default:
      return ast;
  }
}
function omitStandaloneName(ast) {
  switch (ast.type) {
    case "ENUM":
      return ast;
    default:
      return { ...ast, standaloneName: void 0 };
  }
}

// src/parser.ts
import { findKey, includes, isPlainObject as isPlainObject4, map, memoize as memoize3, omit as omit2 } from "lodash";
import { format as format2 } from "util";

// src/typesOfSchema.ts
import { isPlainObject as isPlainObject3 } from "lodash";
function typesOfSchema(schema) {
  if (schema.tsType) {
    return ["CUSTOM_TYPE"];
  }
  const matchedTypes = [];
  for (const [schemaType, f] of Object.entries(matchers)) {
    if (f(schema)) {
      matchedTypes.push(schemaType);
    }
  }
  if (!matchedTypes.length) {
    return ["UNNAMED_SCHEMA"];
  }
  return matchedTypes;
}
var matchers = {
  ALL_OF(schema) {
    return "allOf" in schema;
  },
  ANY(schema) {
    if (Object.keys(schema).length === 0) {
      return true;
    }
    return schema.type === "any";
  },
  ANY_OF(schema) {
    return "anyOf" in schema;
  },
  BOOLEAN(schema) {
    if ("enum" in schema) {
      return false;
    }
    if (schema.type === "boolean") {
      return true;
    }
    if (!isCompound(schema) && typeof schema.default === "boolean") {
      return true;
    }
    return false;
  },
  CUSTOM_TYPE() {
    return false;
  },
  NAMED_ENUM(schema) {
    return "enum" in schema && "tsEnumNames" in schema;
  },
  NAMED_SCHEMA(schema) {
    return "$id" in schema && ("patternProperties" in schema || "properties" in schema);
  },
  NEVER(schema) {
    return schema === false;
  },
  NULL(schema) {
    return schema.type === "null";
  },
  NUMBER(schema) {
    if ("enum" in schema) {
      return false;
    }
    if (schema.type === "integer" || schema.type === "number") {
      return true;
    }
    if (!isCompound(schema) && typeof schema.default === "number") {
      return true;
    }
    return false;
  },
  OBJECT(schema) {
    return schema.type === "object" && !isPlainObject3(schema.additionalProperties) && !schema.allOf && !schema.anyOf && !schema.oneOf && !schema.patternProperties && !schema.properties && !schema.required;
  },
  ONE_OF(schema) {
    return "oneOf" in schema;
  },
  REFERENCE(schema) {
    return "$ref" in schema;
  },
  STRING(schema) {
    if ("enum" in schema) {
      return false;
    }
    if (schema.type === "string") {
      return true;
    }
    if (!isCompound(schema) && typeof schema.default === "string") {
      return true;
    }
    return false;
  },
  TYPED_ARRAY(schema) {
    if (schema.type && schema.type !== "array") {
      return false;
    }
    return "items" in schema;
  },
  UNION(schema) {
    return Array.isArray(schema.type);
  },
  UNNAMED_ENUM(schema) {
    if ("tsEnumNames" in schema) {
      return false;
    }
    if (schema.type && schema.type !== "boolean" && schema.type !== "integer" && schema.type !== "number" && schema.type !== "string") {
      return false;
    }
    return "enum" in schema;
  },
  UNNAMED_SCHEMA() {
    return false;
  },
  UNTYPED_ARRAY(schema) {
    return schema.type === "array" && !("items" in schema);
  }
};

// src/parser.ts
function parse(schema, options, keyName, processed = /* @__PURE__ */ new Map(), usedNames = /* @__PURE__ */ new Set()) {
  if (isPrimitive(schema)) {
    if (isBoolean(schema)) {
      return parseBooleanSchema(schema, keyName, options);
    }
    return parseLiteral(schema, keyName);
  }
  const types = typesOfSchema(schema);
  if (types.length === 1) {
    const ast2 = parseAsTypeWithCache(schema, types[0], options, keyName, processed, usedNames);
    log("blue", "parser", "Types:", types, "Input:", schema, "Output:", ast2);
    return ast2;
  }
  const ast = parseAsTypeWithCache(
    {
      $id: schema.$id,
      allOf: [],
      description: schema.description,
      title: schema.title
    },
    "ALL_OF",
    options,
    keyName,
    processed,
    usedNames
  );
  ast.params = types.map(
    (type) => (
      // We hoist description (for comment) and id/title (for standaloneName)
      // to the parent intersection type, so we remove it from the children.
      parseAsTypeWithCache(maybeStripNameHints(schema), type, options, keyName, processed, usedNames)
    )
  );
  log("blue", "parser", "Types:", types, "Input:", schema, "Output:", ast);
  return ast;
}
function parseAsTypeWithCache(schema, type, options, keyName, processed = /* @__PURE__ */ new Map(), usedNames = /* @__PURE__ */ new Set()) {
  let cachedTypeMap = processed.get(schema);
  if (!cachedTypeMap) {
    cachedTypeMap = /* @__PURE__ */ new Map();
    processed.set(schema, cachedTypeMap);
  }
  const cachedAST = cachedTypeMap.get(type);
  if (cachedAST) {
    return cachedAST;
  }
  const ast = {};
  cachedTypeMap.set(type, ast);
  return Object.assign(ast, parseNonLiteral(schema, type, options, keyName, processed, usedNames));
}
function parseBooleanSchema(schema, keyName, options) {
  if (schema) {
    return {
      keyName,
      type: options.unknownAny ? "UNKNOWN" : "ANY"
    };
  }
  return {
    keyName,
    type: "NEVER"
  };
}
function parseLiteral(schema, keyName) {
  return {
    keyName,
    params: schema,
    type: "LITERAL"
  };
}
function parseNonLiteral(schema, type, options, keyName, processed, usedNames) {
  const definitions = getDefinitionsMemoized(getRootSchema(schema));
  const keyNameFromDefinition = findKey(definitions, (_) => _ === schema);
  switch (type) {
    case "ALL_OF":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        params: schema.allOf.map((_) => parse(_, options, void 0, processed, usedNames)),
        type: "INTERSECTION"
      };
    case "ANY":
      return {
        ...options.unknownAny ? T_UNKNOWN : T_ANY,
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames)
      };
    case "ANY_OF":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        params: schema.anyOf.map((_) => parse(_, options, void 0, processed, usedNames)),
        type: "UNION"
      };
    case "BOOLEAN":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "BOOLEAN"
      };
    case "CUSTOM_TYPE":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        params: schema.tsType,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "CUSTOM_TYPE"
      };
    case "NAMED_ENUM":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition ?? keyName, usedNames),
        params: schema.enum.map((_, n) => ({
          ast: parseLiteral(_, void 0),
          keyName: schema.tsEnumNames[n]
        })),
        type: "ENUM"
      };
    case "NAMED_SCHEMA":
      return newInterface(schema, options, processed, usedNames, keyName);
    case "NEVER":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "NEVER"
      };
    case "NULL":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "NULL"
      };
    case "NUMBER":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "NUMBER"
      };
    case "OBJECT":
      return {
        comment: schema.description,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "OBJECT",
        deprecated: schema.deprecated
      };
    case "ONE_OF":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        params: schema.oneOf.map((_) => parse(_, options, void 0, processed, usedNames)),
        type: "UNION"
      };
    case "REFERENCE":
      throw Error(format2("Refs should have been resolved by the resolver!", schema));
    case "STRING":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "STRING"
      };
    case "TYPED_ARRAY":
      if (Array.isArray(schema.items)) {
        const minItems2 = schema.minItems;
        const maxItems2 = schema.maxItems;
        const arrayType = {
          comment: schema.description,
          deprecated: schema.deprecated,
          keyName,
          maxItems: maxItems2,
          minItems: minItems2,
          standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
          params: schema.items.map((_) => parse(_, options, void 0, processed, usedNames)),
          type: "TUPLE"
        };
        if (schema.additionalItems === true) {
          arrayType.spreadParam = options.unknownAny ? T_UNKNOWN : T_ANY;
        } else if (schema.additionalItems) {
          arrayType.spreadParam = parse(schema.additionalItems, options, void 0, processed, usedNames);
        }
        return arrayType;
      } else {
        return {
          comment: schema.description,
          deprecated: schema.deprecated,
          keyName,
          standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
          params: parse(schema.items, options, `{keyNameFromDefinition}Items`, processed, usedNames),
          type: "ARRAY"
        };
      }
    case "UNION":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        params: schema.type.map((type2) => {
          const member = { ...omit2(schema, "$id", "description", "title"), type: type2 };
          return parse(maybeStripDefault(member), options, void 0, processed, usedNames);
        }),
        type: "UNION"
      };
    case "UNNAMED_ENUM":
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        params: schema.enum.map((_) => parseLiteral(_, void 0)),
        type: "UNION"
      };
    case "UNNAMED_SCHEMA":
      return newInterface(schema, options, processed, usedNames, keyName, keyNameFromDefinition);
    case "UNTYPED_ARRAY":
      const minItems = schema.minItems;
      const maxItems = typeof schema.maxItems === "number" ? schema.maxItems : -1;
      const params = options.unknownAny ? T_UNKNOWN : T_ANY;
      if (minItems > 0 || maxItems >= 0) {
        return {
          comment: schema.description,
          deprecated: schema.deprecated,
          keyName,
          maxItems: schema.maxItems,
          minItems,
          // create a tuple of length N
          params: Array(Math.max(maxItems, minItems) || 0).fill(params),
          // if there is no maximum, then add a spread item to collect the rest
          spreadParam: maxItems >= 0 ? void 0 : params,
          standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
          type: "TUPLE"
        };
      }
      return {
        comment: schema.description,
        deprecated: schema.deprecated,
        keyName,
        params,
        standaloneName: standaloneName(schema, keyNameFromDefinition, usedNames),
        type: "ARRAY"
      };
  }
}
function standaloneName(schema, keyNameFromDefinition, usedNames) {
  const name = schema.title || schema.$id || keyNameFromDefinition;
  if (name) {
    return generateName(name, usedNames);
  }
}
function newInterface(schema, options, processed, usedNames, keyName, keyNameFromDefinition) {
  const name = standaloneName(schema, keyNameFromDefinition, usedNames);
  return {
    comment: schema.description,
    deprecated: schema.deprecated,
    keyName,
    params: parseSchema(schema, options, processed, usedNames, name),
    standaloneName: name,
    superTypes: parseSuperTypes(schema, options, processed, usedNames),
    type: "INTERFACE"
  };
}
function parseSuperTypes(schema, options, processed, usedNames) {
  const superTypes = schema.extends;
  if (!superTypes) {
    return [];
  }
  return superTypes.map((_) => parse(_, options, void 0, processed, usedNames));
}
function parseSchema(schema, options, processed, usedNames, parentSchemaName) {
  let asts = map(schema.properties, (value, key) => ({
    ast: parse(value, options, key, processed, usedNames),
    isPatternProperty: false,
    isRequired: includes(schema.required || [], key),
    isUnreachableDefinition: false,
    keyName: key
  }));
  let singlePatternProperty = false;
  if (schema.patternProperties) {
    singlePatternProperty = !schema.additionalProperties && Object.keys(schema.patternProperties).length === 1;
    asts = asts.concat(
      map(schema.patternProperties, (value, key) => {
        const ast = parse(value, options, key, processed, usedNames);
        const comment = `This interface was referenced by \`${parentSchemaName}\`'s JSON-Schema definition
via the \`patternProperty\` "${key}".`;
        ast.comment = ast.comment ? `${ast.comment}

${comment}` : comment;
        return {
          ast,
          isPatternProperty: !singlePatternProperty,
          isRequired: singlePatternProperty || includes(schema.required || [], key),
          isUnreachableDefinition: false,
          keyName: singlePatternProperty ? "[k: string]" : key
        };
      })
    );
  }
  if (options.unreachableDefinitions) {
    asts = asts.concat(
      map(schema.$defs, (value, key) => {
        const ast = parse(value, options, key, processed, usedNames);
        const comment = `This interface was referenced by \`${parentSchemaName}\`'s JSON-Schema
via the \`definition\` "${key}".`;
        ast.comment = ast.comment ? `${ast.comment}

${comment}` : comment;
        return {
          ast,
          isPatternProperty: false,
          isRequired: includes(schema.required || [], key),
          isUnreachableDefinition: true,
          keyName: key
        };
      })
    );
  }
  switch (schema.additionalProperties) {
    case void 0:
    case true:
      if (singlePatternProperty) {
        return asts;
      }
      return asts.concat({
        ast: options.unknownAny ? T_UNKNOWN_ADDITIONAL_PROPERTIES : T_ANY_ADDITIONAL_PROPERTIES,
        isPatternProperty: false,
        isRequired: true,
        isUnreachableDefinition: false,
        keyName: "[k: string]"
      });
    case false:
      return asts;
    default:
      return asts.concat({
        ast: parse(schema.additionalProperties, options, "[k: string]", processed, usedNames),
        isPatternProperty: false,
        isRequired: true,
        isUnreachableDefinition: false,
        keyName: "[k: string]"
      });
  }
}
function getDefinitions(schema, isSchema = true, processed = /* @__PURE__ */ new Set()) {
  if (processed.has(schema)) {
    return {};
  }
  processed.add(schema);
  if (Array.isArray(schema)) {
    return schema.reduce(
      (prev, cur) => ({
        ...prev,
        ...getDefinitions(cur, false, processed)
      }),
      {}
    );
  }
  if (isPlainObject4(schema)) {
    return {
      ...isSchema && hasDefinitions(schema) ? schema.$defs : {},
      ...Object.keys(schema).reduce(
        (prev, cur) => ({
          ...prev,
          ...getDefinitions(schema[cur], false, processed)
        }),
        {}
      )
    };
  }
  return {};
}
var getDefinitionsMemoized = memoize3(getDefinitions);
function hasDefinitions(schema) {
  return "$defs" in schema;
}

// src/resolver.ts
import $RefParser from "@bcherny/json-schema-ref-parser";
async function dereference(schema, { cwd, $refOptions }) {
  log("green", "dereferencer", "Dereferencing input schema:", cwd, schema);
  const parser = new $RefParser();
  const dereferencedPaths = /* @__PURE__ */ new WeakMap();
  const dereferencedSchema = await parser.dereference(cwd, schema, {
    ...$refOptions,
    dereference: {
      ...$refOptions.dereference,
      onDereference($ref, schema2) {
        dereferencedPaths.set(schema2, $ref);
      }
    }
  });
  return { dereferencedPaths, dereferencedSchema };
}

// src/validator.ts
var rules2 = /* @__PURE__ */ new Map();
rules2.set("Enum members and tsEnumNames must be of the same length", (schema) => {
  if (schema.enum && schema.tsEnumNames && schema.enum.length !== schema.tsEnumNames.length) {
    return false;
  }
});
rules2.set("tsEnumNames must be an array of strings", (schema) => {
  if (schema.tsEnumNames && schema.tsEnumNames.some((_) => typeof _ !== "string")) {
    return false;
  }
});
rules2.set("When both maxItems and minItems are present, maxItems >= minItems", (schema) => {
  const { maxItems, minItems } = schema;
  if (typeof maxItems === "number" && typeof minItems === "number") {
    return maxItems >= minItems;
  }
});
rules2.set("When maxItems exists, maxItems >= 0", (schema) => {
  const { maxItems } = schema;
  if (typeof maxItems === "number") {
    return maxItems >= 0;
  }
});
rules2.set("When minItems exists, minItems >= 0", (schema) => {
  const { minItems } = schema;
  if (typeof minItems === "number") {
    return minItems >= 0;
  }
});
rules2.set("deprecated must be a boolean", (schema) => {
  const typeOfDeprecated = typeof schema.deprecated;
  return typeOfDeprecated === "boolean" || typeOfDeprecated === "undefined";
});
function validate(schema, filename) {
  const errors = [];
  rules2.forEach((rule, ruleName) => {
    traverse(schema, (schema2, key) => {
      if (rule(schema2) === false) {
        errors.push(`Error at key "${key}" in file "${filename}": ${ruleName}`);
      }
      return schema2;
    });
  });
  return errors;
}

// src/index.ts
import { isDeepStrictEqual as isDeepStrictEqual2 } from "util";

// src/linker.ts
import { isPlainObject as isPlainObject5 } from "lodash";
function link(schema, parent = null) {
  if (!Array.isArray(schema) && !isPlainObject5(schema)) {
    return schema;
  }
  if (schema.hasOwnProperty(Parent)) {
    return schema;
  }
  Object.defineProperty(schema, Parent, {
    enumerable: false,
    value: parent,
    writable: false
  });
  if (Array.isArray(schema)) {
    schema.forEach((child) => link(child, schema));
  }
  for (const key in schema) {
    link(schema[key], schema);
  }
  return schema;
}

// src/optionValidator.ts
function validateOptions({ maxItems }) {
  if (maxItems !== void 0 && maxItems < -1) {
    throw RangeError(`Expected options.maxItems to be >= -1, but was given ${maxItems}.`);
  }
}

// src/index.ts
var DEFAULT_OPTIONS = {
  $refOptions: {},
  additionalProperties: true,
  // TODO: default to empty schema (as per spec) instead
  bannerComment: `/* eslint-disable */
/**
* This file was automatically generated by json-schema-to-typescript.
* DO NOT MODIFY IT BY HAND. Instead, modify the source JSONSchema file,
* and run json-schema-to-typescript to regenerate this file.
*/`,
  cwd: process.cwd(),
  declareExternallyReferenced: true,
  enableConstEnums: true,
  format: true,
  ignoreMinAndMaxItems: false,
  maxItems: 20,
  strictIndexSignatures: false,
  style: {
    bracketSpacing: false,
    printWidth: 120,
    semi: true,
    singleQuote: false,
    tabWidth: 2,
    trailingComma: "none",
    useTabs: false
  },
  unreachableDefinitions: false,
  unknownAny: true
};
function compileFromFile(filename, options = DEFAULT_OPTIONS) {
  const contents = Try(
    () => readFileSync(filename),
    () => {
      throw new ReferenceError(`Unable to read file "${filename}"`);
    }
  );
  const schema = Try(
    () => JSON.parse(contents.toString()),
    () => {
      throw new TypeError(`Error parsing JSON in file "${filename}"`);
    }
  );
  return compile(schema, stripExtension(filename), { cwd: dirname2(filename), ...options });
}
async function compile(schema, name, options = {}) {
  validateOptions(options);
  const _options = merge({}, DEFAULT_OPTIONS, options);
  const start = Date.now();
  function time() {
    return `(${Date.now() - start}ms)`;
  }
  if (!endsWith(_options.cwd, "/")) {
    _options.cwd += "/";
  }
  const _schema = cloneDeep(schema);
  const { dereferencedPaths, dereferencedSchema } = await dereference(_schema, _options);
  if (process.env.VERBOSE) {
    if (isDeepStrictEqual2(_schema, dereferencedSchema)) {
      log("green", "dereferencer", time(), "\u2705 No change");
    } else {
      log("green", "dereferencer", time(), "\u2705 Result:", dereferencedSchema);
    }
  }
  const linked = link(dereferencedSchema);
  if (process.env.VERBOSE) {
    log("green", "linker", time(), "\u2705 No change");
  }
  const errors = validate(linked, name);
  if (errors.length) {
    errors.forEach((_) => error(_));
    throw new ValidationError();
  }
  if (process.env.VERBOSE) {
    log("green", "validator", time(), "\u2705 No change");
  }
  const normalized = normalize2(linked, dereferencedPaths, name, _options);
  log("yellow", "normalizer", time(), "\u2705 Result:", normalized);
  const parsed = parse(normalized, _options);
  log("blue", "parser", time(), "\u2705 Result:", parsed);
  const optimized = optimize(parsed, _options);
  log("cyan", "optimizer", time(), "\u2705 Result:", optimized);
  const generated = generate(optimized, _options);
  log("magenta", "generator", time(), "\u2705 Result:", generated);
  const formatted = format(generated, _options);
  log("white", "formatter", time(), "\u2705 Result:", formatted);
  return formatted;
}
var ValidationError = class extends Error {
};

export {
  __require,
  __commonJS,
  error,
  pathTransform,
  DEFAULT_OPTIONS,
  compileFromFile,
  compile,
  ValidationError
};
